<?php

namespace App\Http\Controllers;

use App\Http\Requests\DraftOrderValidation;
use App\Models\GetPackCount;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Log;

class OrderController extends Controller
{
    private $shop;
    private $packCountObj;
    private $subscriptionProductId;
    private $subscriptionVariantId;

    /**
     * Create a new controller instance.
     *
     * @return void
     */
    public function __construct() {

        $this->packCountObj = new GetPackCount();
        $this->subscriptionProductId = env('SUBSCRIPTION_PRODUCT_ID', true);
        $this->subscriptionVariantId = env('SUBSCRIPTION_VARIANT_ID', true);
    }

    /**
     * To create the order view
     */
    public function createOrder($status = []) {

        $products  = $this->setProductDetails();
        $jsonProductDetails = json_encode($products);
        $status = json_encode($status);

        return view('order', compact('jsonProductDetails'), compact('status'));
    }

    /**
     * Create an order in Shopify
     */
    public function saveOrder(DraftOrderValidation $req) {

        $customerResponse = '';
        $setOrderRequest = $this->setOrderRequest($req);
        $setCustomerRequest = $this->setCustomerRequest($req);
        $shop = Auth::user();

        $customOrderDetails = [];
        $params['order'] = $setOrderRequest;

        $products  = $this->setProductDetails();
        $jsonProductDetails = json_encode($products);

        $result = $shop->api()->rest('POST', htmlentities('/admin/orders.json'), $params);
        $result = (object)$result;

        if (!isset($result->body->order) || empty($result->body->order->id)) {
            $errorMessage = "";
            if (isset($result->errors)) {
                $errorMessage = json_encode($result->errors);
            } else if (isset($result->error)) {
                $errorMessage = json_encode($result->error);
            } else {
                $errorMessage = "Failed to create the order in Shopify";
            }
            Log::channel('orderLog')->info("saveOrder: Failed to create the order in Shopify, 
            OrderId:".$order_id.", Error:".$errorMessage);
            $orderStatus = "failed";
            $errorResponse = "Failed to create the order in Shopify";
            return view('order', compact('jsonProductDetails'), compact('orderStatus'));
        }

        $customerUpdateDetails = [];

        if (!empty($result->body->order->customer->id)) {

            $customerUpdateDetails['id'] = $result->body->order->customer->id;
            $customerUpdateDetails['first_name'] = !empty($req->firstName)
                ? $req->firstName : '';
            $customerUpdateDetails['last_name'] = !empty($req->lastName)
                ? $req->lastName : '';

            $updateParam['customer'] = $customerUpdateDetails;

            $customerResponse = $shop->api()->rest(
                'PUT',
                htmlentities('/admin/customers/' . $result->body->order->customer->id . '.json'),
                $updateParam
            );
            if (!isset($customerResponse->body->customer) || empty($customerResponse->body->customer->id)) {
                $errorMessage = "";
                if (isset($customerResponse->errors)) {
                    $errorMessage = json_encode($customerResponse->errors);
                } else if (isset($result->error)) {
                    $errorMessage = json_encode($customerResponse->error);
                } else {
                    $errorMessage = "Failed to update the customer details in Shopify";
                }
                Log::channel('orderLog')->info("saveOrder: Failed to update the customer details in Shopify, 
                CustomerId:".$result->body->order->customer->id.", Error:".$errorMessage);
                // $orderStatus = "failed";
                // $errorResponse = "Failed to update the customer details in Shopify";
                // return view('order', compact('jsonProductDetails'), compact('orderStatus'));
            }
        }

        $orderStatus = "success";
        return view('order', compact('jsonProductDetails'), compact('orderStatus'));
    }

    /**
     * Set customer request
     */
    public function setCustomerRequest($reqObj) {

        $customerRequest['customer']['first_name'] = $reqObj->firstName;
        $customerRequest['customer']['last_name']  = $reqObj->lastName;
        $customerRequest['customer']['email']      = $reqObj->email;
        return $customerRequest;
    }

    /**
     * Set order create request
     */
    public function setOrderRequest($reqObj) {

        $validated = $reqObj->validated();
        $request = [];
        $lineProperties = [];
        $prod  = $this->setProductDetails();
        $i = 1;
        $price = 0;
        $unit_price = '';
        
        $packDetails = [];
        foreach ($reqObj['prod-id'] as $key => $productId) {
            $propertyValue = '';

            if ($i < 10) {
                $propertyKey = 'Vitamin0' . $i++;
            } else if ($i >= 10) {
                $propertyKey = 'Vitamin' . $i++;
            }
            if (isset($prod['products'][$productId])) {

                if (isset($reqObj['prod-variant-id'][$key])) {
                    $variantId = $reqObj['prod-variant-id'][$key];
                    $sku = $prod["products"][$productId]['variants'][$variantId]['sku'];

                    $temp = [];
                    $temp['bulkID'] = $sku;
                    $temp['qty'] = $reqObj['prod-qty'][$key];
                    $packDetails[] = $temp;

                    if (isset($reqObj['prod-qty'][$key])) {

                        $propertyValue .= $prod['products'][$productId]['title'] . ' | ' .
                            $prod["products"][$productId]['variants'][$variantId]['title'] . ' | ' .
                            $reqObj['prod-qty'][$key] . ' | ' .
                            $prod["products"][$productId]['variants'][$variantId]['id'] . ' | ' .
                            $prod["products"][$productId]['variants'][$variantId]['sku'];

                        $lineProperties[] = ['name' => $propertyKey, 'value' => $propertyValue];
                        $unit_price = ($prod["products"][$productId]['variants'][$variantId]['price']) * $reqObj['prod-qty'][$key];
                        $price += $unit_price;
                    }
                }
            }
        }

        $packData = $this->getPackCount($packDetails);
        Log::channel('orderLog')->info("setOrderRequest: ProductPackCount => " .json_encode($packData));

        $productPackCount = '';
        if (isset($packData) || !empty($packData)) {
            $productPackCount = !empty($packData['Pack']) ? "{$packData['Pack']}" : 1;
            $lineProperties[] = ['name' => 'Packs', 'value' => $productPackCount];
        }

        $request['customer']['first_name'] = $reqObj->firstName;
        $request['customer']['last_name'] = $reqObj->lastName;
        $request['customer']['email'] = $reqObj->email;
        $request['email'] = $reqObj->email;
        $request['tags'] = "custom-order-tool-created,".$reqObj->shOrderTags;

        $request['line_items'] = array(
            array(
                'title' => "Your Monthly Subscription",
                'product_id' => $this->subscriptionProductId,
                'variant_id' => $this->subscriptionVariantId,
                'price' => $price,
                'quantity' => '1',
                'properties' => $lineProperties
            )
        );

        if (empty($reqObj->shipAddress1)) {
            $customerDetails = $this->getCustomerAddress($reqObj->email);

            if (!empty($customerDetails)) {
                $custDetails = $customerDetails->body->customers[0];
                $customerId = $custDetails->id;
                $address1 = !empty($custDetails->default_address->address1) ? $custDetails->default_address->address1 : null;
                $address2 = !empty($custDetails->default_address->address2) ? $custDetails->default_address->address2 : null;
                $city = !empty($custDetails->default_address->city) ? $custDetails->default_address->city : null;
                $country = !empty($custDetails->default_address->country) ? $custDetails->default_address->country : null;
                $firstName = !empty($custDetails->default_address->first_name) ? $custDetails->default_address->first_name : null;
                $lastName = !empty($custDetails->default_address->last_name) ? $custDetails->default_address->last_name : null;
                $province = !empty($custDetails->default_address->province_code) ? $custDetails->default_address->province_code : null;
                $zip = !empty($custDetails->default_address->zip) ? $custDetails->default_address->zip : null;

                $request['shipping_address'] = array(
                    "address1" => $address1,
                    "address2" => $address2,
                    "city" => $city,
                    "country" => $country,
                    "first_name" => $firstName,
                    "last_name" => $lastName,
                    "province" => $province,
                    "zip" => $zip,
                );
            } else {
                $request['billing_address'] = null;
            }
        } else {
            $request['shipping_address'] = array(
                "address1" => $reqObj->shipAddress1,
                "address2" => $reqObj->shipAddress2,
                "city" => $reqObj->shipCity,
                "country" => $reqObj->shipCountry,
                "first_name" => $reqObj->shipFirstname,
                "last_name" => $reqObj->shipLastname,
                "province" => $reqObj->shipProvince,
                "zip" => $reqObj->shipZip,
            );
        }
        if (empty($reqObj->billAddress1)) {

            $customerDetails = $this->getCustomerAddress($reqObj->email);

            if (!empty($customerDetails)) {
                $custDetails = $customerDetails->body->customers[0];
                $customerId = $custDetails->id;
                $address1 = !empty($custDetails->default_address->address1) ? $custDetails->default_address->address1 : null;
                $address2 = !empty($custDetails->default_address->address2) ? $custDetails->default_address->address2 : null;
                $city = !empty($custDetails->default_address->city) ? $custDetails->default_address->city : null;
                $country = !empty($custDetails->default_address->country) ? $custDetails->default_address->country : null;
                $firstName = !empty($custDetails->default_address->first_name) ? $custDetails->default_address->first_name : null;
                $lastName = !empty($custDetails->default_address->last_name) ? $custDetails->default_address->last_name : null;
                $province = !empty($custDetails->default_address->province_code) ? $custDetails->default_address->province_code : null;
                $zip = !empty($custDetails->default_address->zip) ? $custDetails->default_address->zip : null;

                $request['billing_address'] = array(
                    "address1" => $address1,
                    "address2" => $address2,
                    "city" => $city,
                    "country" => $country,
                    "first_name" => $firstName,
                    "last_name" =>  $lastName,
                    "province" => $province,
                    "zip" =>  $zip,
                );
            } else {

                $request['billing_address'] = null;
            }
        } else {
            $request['billing_address'] = array(
                "address1" => $reqObj->billAddress1,
                "address2" => $reqObj->billAddress2,
                "city" => $reqObj->billCity,
                "country" => $reqObj->billCountry,
                "first_name" => $reqObj->billFirstname,
                "last_name" => $reqObj->billLastname,
                "province" => $reqObj->billState,
                "zip" => $reqObj->billZip,
            );
        }
        return $request;
    }

    /**
     * Get customer address call
     */
    public function getCustomerAddress($email) {

        $customerObj = Auth::user();
        $customerEmail = $email;
        $params = ["query" => "email:{$customerEmail}"];
        $result = $customerObj->api()->rest('GET', htmlentities('/admin/customers/search.json'), $params);

        if (!isset($result->body->customers) && empty($result->body->customers)) {
            Log::channel('orderLog')->info('getCustomerAddress: Failed to get the customer details from Shopify, Email:'.$customerEmail);
            return null;
        }

        return $result;
    }

    /**
     * Get customer address ajax call
     */
    public function getCustomerAddress2(Request $req) {

        $customerDetails = [];
        $shop = Auth::user();
        $limit = 250;
        $customerEmail = $req->email;
        $params = ["limit" => $limit, "query" => "email:{$customerEmail}"];
        $result = $shop->api()->rest('GET', htmlentities('/admin/customers/search.json'), $params);

        $result = (object)$result;

        if (!isset($result->body->customers) && empty($result->body->customers)) {
            Log::channel('orderLog')->info('getCustomerAddress2: Failed to get the customer details from Shopify, Email:'.$customerEmail);
            return null;
        }

        foreach ($result->body->customers as $customer) {

            if (trim(strtolower($customer['email'])) == trim(strtolower($customerEmail))) {

                $customerDetails['body']['customers'][0] = $customer;
                return json_encode($customerDetails);
            }
        }

        return null;
    }

    /**
     * To get Shopify product details
     */
    private function setProductDetails() {

        $shVariants = [];
        $shProducts['products'] = [];
        $shProducts  = [];
        $limit = 250;
        $nextLink = '';
        $previousLink = '';
        $prodCount = 0;

        $shop    = Auth::user();

        do {

            try {

                if (empty($previousLink) && empty($nextLink)) {

                    $productDetails = $shop->api()->rest(
                        'GET',
                        '/admin/products.json',
                        ["published_status" => "published", "fields" =>
                        "id,title,variants", "limit" => $limit]
                    );

                    $productDetails = (object)$productDetails;

                    $previousLink = !empty($productDetails->link->previous) ?
                        $productDetails->link->previous : '';
                    $nextLink = !empty($productDetails->link->next) ?
                        $productDetails->link->next : '';
                } elseif (!empty($nextLink)) {

                    $params = [
                        "fields" => "id,title,variants", "limit" => $limit,
                        'page_info' => $nextLink
                    ];
                    $productDetails = $shop->api()->rest('GET', '/admin/products.json', $params);
                    $productDetails = (object)$productDetails;

                    $previousLink = !empty($productDetails->link->previous) ?
                        $productDetails->link->previous : '';
                    $nextLink = !empty($productDetails->link->next) ?
                        $productDetails->link->next : '';
                }
            } catch (Exception $e) {
                Log::channel('orderLog')->info('setProductDetails: An error occurred when fetching the products from Shopify');
                echo PHP_EOL . "An error occurred when fetching the products from Shopify and the error is -  ". $e->getMessage();
                die();
            }

            $productDetails = $productDetails->body;

            if (!empty($productDetails)) {

                $subscription_product = array('1809168695367', '4359186350162');

                foreach ($productDetails->products as $products) {
                    if (in_array($products->id, $subscription_product)) {
                        continue;
                    }


                    foreach ($products->variants as $variants) {
                        $prodCount++;

                        $shVariants[$variants->id] = [
                            'id' => $variants->id,
                            'product_id' => $variants->product_id,
                            'title' => strip_tags($variants->title),
                            'sku' => $variants->sku,
                            'price' => $variants->price
                        ];
                    }
                    $shProducts['products'][$products->id] = [
                        'id' => $products->id,
                        'title' => strip_tags($products->title), 'variants' => $shVariants
                    ];
                    $shVariants = [];
                }
            }
            $count = !empty($productDetails) ? count($productDetails->products) : 0;
            if (empty($nextLink)) {
                break;
            }
        } while ($limit == $count);

        return $shProducts;
    }

    /**
     * Get product pack count
     */
    public function getPackCount($requestData) {

        $packRequest['ProductItems'] = array();
        $errorMessage = '';

        if (empty($requestData) || !isset($requestData)) {

            $returnData = [
                'error' => 'Empty parameters'
            ];
            Log::channel('orderLog')->info('getPackCount: Get pack count request parameters are empty');
            return $this->returnData(600, "error", $returnData);
        }
        Log::channel('orderLog')->info('getPackCount: Reuqest => '.json_encode($requestData));

        foreach ($requestData as $data) {

            $bulkID = isset($data['bulkID']) ? $data['bulkID'] : '';
            $qty = isset($data['qty']) ? $data['qty'] : '';

            if (empty($bulkID)) {
               
                $returnData = [
                    'error' => 'Empty parameter value - bulkID'
                ];
                Log::channel('orderLog')->info('getPackCount: Empty parameter value - bulkID');
                return $this->returnData(600, "error", $returnData);
            }

            if (empty($qty)) {

                $returnData = [
                    'error' => 'Empty parameter value - qty'
                ];
                Log::channel('orderLog')->info('getPackCount: Empty parameter value - qty');
                return $this->returnData(600, "error", $returnData);
            }

            if ($bulkID == 'vitamin-pack') {
                continue;
            }

            $tempPackRequest['BulkItem'] = $bulkID;
            $tempPackRequest['Quantity'] = $qty;
            $packRequest['ProductItems'][] = $tempPackRequest;
        }

        if (empty($packRequest['ProductItems'])) {

            $returnData = [
                'error' => 'Empty ProductItems'
            ];
            Log::channel('orderLog')->info('getPackCount: Empty ProductItems');
            return $this->returnData(600, "error", $returnData);
        }

        $returnASCII = $this->encode_items($packRequest);
        //$packDetails = $this->packCountObj->getPackCount($returnASCII);
        $packDetails = "count";

        if (strpos($packDetails, 'Packs') !== false) {
            $stripData = explode("Packs", $packDetails);
            $packData = $stripData[1];
            $packData = ltrim($packData, ":");
            $packData = rtrim($packData, "}");

            $returnData = [
                'Pack' => $packData,
            ];
            return $returnData;
        } else {

            $errorMessage = 'No Pack details';
            return $errorMessage;
        }
    }

    /**
     * Encode items
     */
    public function encode_items($array) {

        foreach ($array as $key => $value) {
            if (is_array($value)) {
                $array[$key] = $this->encode_items($value);
            } else {
                $array[$key] = mb_convert_encoding($value, 'UTF-8');
            }
        }

        return $array;
    }

    /**
     * Return JSON response
     */
    public function returnData($status, $message, $body) {

        $returnData = array(
            'status' => $status,
            'messsage' => $message,
            'body' => $body
        );
        return response()->json($returnData);
    }
}
